/*
 * 쐬: 2004/01/02
 *
 */
package com.muimi.tomcat;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.ui.DebugUITools;
import org.eclipse.debug.ui.IDebugUIConstants;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.debug.ui.JavaUISourceLocator;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.preference.IPreferenceStore;


/**
 * @author miya
 *
 */
public class TomcatLauncher
	implements IJavaLaunchConfigurationConstants {

	private String tomcatHome;// = "C:/app/jakarta-tomcat-4.1.27";
	private String mode = ILaunchManager.DEBUG_MODE;
	private String arg = "start";


	public void launch() throws CoreException {

		IPreferenceStore store = TomcatPlugin.getDefault().getPreferenceStore();
		tomcatHome = store.getString("tomcat_home");

		ILaunchConfiguration configuration = createConfiguration();

		DebugUITools.launch(configuration, mode);
	}



	private ILaunchConfiguration createConfiguration() throws CoreException {

		//VKLaunchConfiguration쐬Bɂ̂͏
		ILaunchConfigurationWorkingCopy workingCopy =
			createNewConfigurationWorkingCopy("tomcat launch");


		//LaunchConfigurationN\_CAOɕ\Ȃ
		//쐬LaunchConfigurationmFꍇ̓RgAEg
		workingCopy.setAttribute(IDebugUIConstants.ATTR_PRIVATE, true);


		//JRE̎wBł̓ftHg𗘗p
		IVMInstall jre = JavaRuntime.getDefaultVMInstall();
		workingCopy.setAttribute(ATTR_VM_INSTALL_NAME, jre.getName());
		workingCopy.setAttribute(
			ATTR_VM_INSTALL_TYPE,
			jre.getVMInstallType().getId());


		//MainNXAAul
		workingCopy.setAttribute(
			ATTR_MAIN_TYPE_NAME,
			"org.apache.catalina.startup.Bootstrap");
		workingCopy.setAttribute(ATTR_PROGRAM_ARGUMENTS, arg);
		workingCopy.setAttribute(
			ATTR_VM_ARGUMENTS,
			"-Djava.endorsed.dirs=\"..\\common\\endorsed\" " +
			"-Dcatalina.base=\"..\" " +
			"-Dcatalina.home=\"..\" " +
			"-Djava.io.tmpdir=\"..\\temp\""
			);


		//NXpX̎w
		File jdkHome = jre.getInstallLocation();
		IPath toolsPath =
			new Path(jdkHome.getAbsolutePath()).append("lib").append(
				"tools.jar");
		IRuntimeClasspathEntry toolsEntry =
			JavaRuntime.newArchiveRuntimeClasspathEntry(toolsPath);
		toolsEntry.setClasspathProperty(IRuntimeClasspathEntry.USER_CLASSES);

		IPath bootstrapPath =
			new Path(tomcatHome).append("bin").append("bootstrap.jar");
		IRuntimeClasspathEntry bootstrapEntry =
			JavaRuntime.newArchiveRuntimeClasspathEntry(bootstrapPath);
		JavaRuntime.newVariableRuntimeClasspathEntry(bootstrapPath);
		bootstrapEntry.setClasspathProperty(
			IRuntimeClasspathEntry.USER_CLASSES);

		IPath systemLibsPath = new Path(JavaRuntime.JRE_CONTAINER);
		IRuntimeClasspathEntry systemLibsEntry =
			JavaRuntime.newRuntimeContainerClasspathEntry(
				systemLibsPath,
				IRuntimeClasspathEntry.STANDARD_CLASSES);

		List classpath = new ArrayList();
		classpath.add(toolsEntry.getMemento());
		classpath.add(bootstrapEntry.getMemento());
		classpath.add(systemLibsEntry.getMemento());
		//classpath.addAll(getProjectClassPathMemeonts());

		workingCopy.setAttribute(ATTR_CLASSPATH, classpath);
		workingCopy.setAttribute(ATTR_DEFAULT_CLASSPATH, false);


		//\[XpX̎w
		List sourcePathList = new ArrayList();
		sourcePathList.add(systemLibsEntry.getMemento());
		sourcePathList.addAll(getProjectClassPathMemeonts());
		workingCopy.setAttribute(ATTR_SOURCE_PATH, sourcePathList);
		workingCopy.setAttribute(ATTR_DEFAULT_SOURCE_PATH, false);
		workingCopy.setAttribute(ILaunchConfiguration.ATTR_SOURCE_LOCATOR_ID, JavaUISourceLocator.ID_PROMPTING_JAVA_SOURCE_LOCATOR);


		//ƃfBNg̎w
		File workingDir = new Path(tomcatHome).append("bin").toFile();
		workingCopy.setAttribute(
			ATTR_WORKING_DIRECTORY,
			workingDir.getAbsolutePath());



		ILaunchConfiguration configuration = workingCopy.doSave();

		return configuration;
	}


	private ILaunchConfigurationWorkingCopy createNewConfigurationWorkingCopy(String name)
		throws CoreException {
		ILaunchManager manager = DebugPlugin.getDefault().getLaunchManager();
		ILaunchConfigurationType type =
			manager.getLaunchConfigurationType(ID_JAVA_APPLICATION);

		ILaunchConfiguration[] configurations =
			manager.getLaunchConfigurations(type);
		for (int i = 0; i < configurations.length; i++) {
			ILaunchConfiguration configuration = configurations[i];
			if (configuration.getName().equals(name)) {
				configuration.delete();
				break;
			}
		}

		ILaunchConfigurationWorkingCopy workingCopy =
			type.newInstance(null, name);
		return workingCopy;
	}

	private List getProjectClassPathMemeonts() throws CoreException {
		ArrayList list = new ArrayList();

		IProject[] projects =
			ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (int i = 0; i < projects.length; i++) {
			IProject project = projects[i];
			if (project.isOpen() && project.hasNature(JavaCore.NATURE_ID)) {
				IJavaProject javaProject = JavaCore.create(project);
				IRuntimeClasspathEntry entry =
					JavaRuntime.newProjectRuntimeClasspathEntry(javaProject);
				entry.setClasspathProperty(IRuntimeClasspathEntry.PROJECT);
				entry.setClasspathProperty(IRuntimeClasspathEntry.USER_CLASSES);
				list.add(entry.getMemento());
			}
		}

		return list;
	}


	/**
	 * @return
	 */
	public String getArg() {
		return arg;
	}

	/**
	 * @param string
	 */
	public void setArg(String string) {
		arg = string;
	}

	/**
	 * @return
	 */
	public String getMode() {
		return mode;
	}

	/**
	 * @param string
	 */
	public void setMode(String string) {
		mode = string;
	}

}
